const { exec } = require('child_process');
const fs = require("fs");
const inquirer = require('inquirer');
const chalk = require("chalk");
const program = require("commander");
const path = require("path");
const axios = require("axios");
const ora = require("ora");
const { version, name : packageName } = require("../package.json");
const { ajaxBaseUrl, gitlabUrl, sshGitlabUrl, httpGitlabUrl, pcUrl, loginInfoSaveFileName, cliName } = require("./config")

// 请求loading
const waitLoading = (fn, message) => async (...args) => {
  const spinner = ora(message);
  spinner.start()
  const result = await fn(...args)
  spinner.succeed()
  return result;
}

const mapActions = {
  cdn: {
    alias: "c",
    description: "发布文件夹到cdn",
    examples: [
      `${cliName} cdn <dirname>`
    ]
  },
  page: {
    alias: "p",
    description: "发布页面",
    examples: [
      `${cliName} page <file-path>`
    ]
  },
  '*': {
    alias: '',
    description: "not found",
    examples: []
  }
}
// program
//   .option('-m, --pagePublishRemark <type>', '页面发布备注信息');

// console.log(program.pagePublishRemark)

Reflect.ownKeys(mapActions).forEach(action => {
  program
    .command(action)
    .alias(mapActions[action].alias)
    .description(mapActions[action].description)
    .action(() => {
      if (action === '*') {
        console.log(mapActions[action].description)
      } else {
        main(action)
      }
    })
})



program
  .on('--help', () => {
    console.log("\n示例")
    Reflect.ownKeys(mapActions).forEach(action => {
      mapActions[action].examples.forEach(example => {
        console.log(`  ${example}`)
      })
    })
  })

program
  .version(version)
  .parse(process.argv)

if (!process.argv.slice(2).length) {
  program.outputHelp();
}






async function main (action) {
  try {
    // await validateVersion(); // 检查版本
    const userInfo = await validateUser(); // 校验用户
    const projectName = await validateGitRemote(); // 检查仓库 cli/publish-platform
    const currentBranch = await validateGitBranch(); // 检查分支
    await validatePush(); // 代码是否提交
    require(path.join(__dirname, action))(userInfo, projectName, currentBranch, ...process.argv.slice(3))
  } catch (err) {
    console.log(chalk.red(err))
  }
}




/**
 * 校验用户信息
 */
async function validateUser () {
  const downloadDirectory = `${process.env[process.platform === 'darwin' ? 'HOME' : 'USERPROFILE']}/.${loginInfoSaveFileName}`;
  if (fs.existsSync(downloadDirectory)) {
    let userInfo = fs.readFileSync(downloadDirectory, 'utf-8');
    try {
      userInfo = JSON.parse(userInfo);
      if (Object.prototype.toString.call(userInfo).toLocaleLowerCase() === "[object object]" && userInfo.name && userInfo.password) {
        const selectUser = await waitLoading(login, "正在校验用户信息...")(userInfo)
        if (!selectUser.status) {
          throw "用户信息错误";
        } else {
          return userInfo;
        }
      } else {
        throw "用户信息错误";
      }
    } catch(err) {
      console.log(chalk.red("登录文件校验错误，请重新登录"));
      fs.unlinkSync(downloadDirectory);
      return validateUser();
    }
  } else {
    const userInfo = await inquirer.prompt([{
      type: 'input',
      message: '请输入用户名',
      name: 'name',
      validate: val => { if (!val) return "用户名不能为空"; return true }
    }, {
      type: 'password',
      message: '请输入密码',
      name: 'password',
      validate: val => { if (!val) return "密码不能为空"; return true }
    }]);
    const selectUser = await waitLoading(login, "登录中...")(userInfo)
    if (!selectUser.status) {
      console.log(chalk.red(selectUser.message));
      return validateUser();
    }
    fs.writeFileSync(downloadDirectory, JSON.stringify(userInfo));
    return userInfo;
  }
}

/**
 * 用户登录
 */
async function login({ name, password }) {
  return axios({
    url: `${ajaxBaseUrl}/login`,
    method: "post",
    data: { name, password }
  }).then(res => res.data)
}

/**
 * 校验远程仓库是gitlab.redcat.im
 */
async function validateGitRemote () {
  return new Promise((resolve, reject) => {
    exec("git remote -v", (err, stdout, stderr) => {
      if (err) {
        reject(err)
      } else {
        // git@gitlab.redcat.im:hujili/zyt-backstage.git
        // http://gitlab.redcat.im/hujili/zyt-backstage.git
        if (stdout.includes(gitlabUrl)) {
          let gitUrl = stdout.split("\n")[1];
          let splitFlag
          if (gitUrl.includes(sshGitlabUrl)) {
            splitFlag = sshGitlabUrl
          } else {
            splitFlag = httpGitlabUrl;
          }
          let projectName = gitUrl.split(splitFlag)[1];
          if (projectName.includes(".git")) { // 兼容 文彪电脑git地址异常，没有.git origin  http://gitlab.redcat.im/cli/publish-platform (push)
            projectName = projectName.split(".git")[0];
          } else {
            projectName = projectName.split(" ")[0];
          }
          resolve(projectName)
        } else {
          reject("当前非公司仓库，禁止发布")
        }
      }
    })
  })
}

/**
 * 检查分支
 */
async function validateGitBranch () {
  return new Promise((resolve, reject) => {
    exec("git branch", (err, stdout, stderr) => {
      if (err) {
        reject(err)
      } else {
        const branchList = stdout.split("\n");
        let currentBranchStr = branchList.find(item => item.includes("*"))
        let currentBranch = currentBranchStr.slice(1).trim()
        const publishBranch = ["daily", "gray", "publish"];
        if (publishBranch.includes(currentBranch)) {
          resolve(currentBranch)
        } else {
          reject('只能发布daily gray publish 分支')
        }
      }
    })
  })
}

/***
 * 检查版本号
 */
async function validateVersion() {
  return new Promise((resolve, reject) => {
    exec(`${cliName} --version`, (err, stdout, stderr) => {
      if (err) {
        reject(err)
      } else {
        axios({
          method: "get",
          url: `${ajaxBaseUrl}/cliConfig/currentVersion`
        }).then(res => res.data).then(({ entry: { current_version, update_log } }) => {
          let old_version = stdout.trim();
          if (current_version === old_version) {
            resolve()
          } else {
            console.log("有版本更新啦")
            console.log(`当前版本 ${old_version}, 最新版${current_version}`)
            console.log("请使用" + chalk.green(`npm i ${packageName} -g`) + "更新到最新版");
            console.log("更新日志" + update_log);
            console.log("详情访问" + chalk.green(`${pcUrl}/#/intro`))
            reject("")
          }
        })
      }
    })
  })
}

/**
 * 检查代码是否提交
 */
async function validatePush() {
  return new Promise((resolve, reject) => {
    exec("git status", (err, stdout, stderr) => {
      if (err) {
        reject(err)
      } else {
        if (stdout.includes('Changes not staged for commit') || stdout.includes('Changes to be committed') || stdout.includes('(use "git push" to publish your local commits)')) {
          reject("发布前，请先提交代码")
        } else {
          resolve()
        }
      }
    })
  })
}